<?php

namespace Assetdocs\Demotable\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Form;
use Assetdocs\Demotable\Http\Requests\DemotableAdminWebRequest;
use Assetdocs\Demotable\Interfaces\DemotableRepositoryInterface;
use Assetdocs\Demotable\Models\Demotable;

/**
 * Admin web controller class.
 */
class DemotableAdminController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'admin.web';

    /**
     * Initialize demotable controller.
     *
     * @param type DemotableRepositoryInterface $demotable
     *
     * @return type
     */
    public function __construct(DemotableRepositoryInterface $demotable)
    {
        $this->middleware('web');
        $this->middleware('auth:admin.web');
        $this->setupTheme(config('theme.themes.admin.theme'), config('theme.themes.admin.layout'));

        $this->repository = $demotable;
        parent::__construct();
    }

    /**
     * Display a list of demotable.
     *
     * @return Response
     */
    public function index(DemotableAdminWebRequest $request)
    {
        $pageLimit = $request->input('pageLimit');
        if ($request->wantsJson()) {
            $demotables  = $this->repository
            ->pushCriteria(new \Assetdocs\Demotable\Repositories\Criteria\DemotableAdminCriteria())
                ->setPresenter('\\Assetdocs\\Demotable\\Repositories\\Presenter\\DemotableListPresenter')
                ->scopeQuery(function ($query) {
                    return $query->orderBy('id', 'DESC');
                })->paginate($pageLimit);

            $demotables['recordsTotal']    = $demotables['meta']['pagination']['total'];
            $demotables['recordsFiltered'] = $demotables['meta']['pagination']['total'];
            $demotables['request']         = $request->all();
            return response()->json($demotables, 200);

        }

        $this   ->theme->prependTitle(trans('demotable::demotable.names').' :: ');
        return $this->theme->of('demotable::admin.demotable.index')->render();
    }

    /**
     * Display demotable.
     *
     * @param Request $request
     * @param Model   $demotable
     *
     * @return Response
     */
    public function show(DemotableAdminWebRequest $request, Demotable $demotable)
    {
        if (!$demotable->exists) {
            return response()->view('demotable::admin.demotable.new', compact('demotable'));
        }

        Form::populate($demotable);
        return response()->view('demotable::admin.demotable.show', compact('demotable'));
    }

    /**
     * Show the form for creating a new demotable.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DemotableAdminWebRequest $request)
    {

        $demotable = $this->repository->newInstance([]);

        Form::populate($demotable);

        return response()->view('demotable::admin.demotable.create', compact('demotable'));

    }

    /**
     * Create new demotable.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DemotableAdminWebRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.web');
            $demotable          = $this->repository->create($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('demotable::demotable.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/demotable/demotable/'.$demotable->getRouteKey())
            ], 201);


        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
            ], 400);
        }
    }

    /**
     * Show demotable for editing.
     *
     * @param Request $request
     * @param Model   $demotable
     *
     * @return Response
     */
    public function edit(DemotableAdminWebRequest $request, Demotable $demotable)
    {
        Form::populate($demotable);
        return  response()->view('demotable::admin.demotable.edit', compact('demotable'));
    }

    /**
     * Update the demotable.
     *
     * @param Request $request
     * @param Model   $demotable
     *
     * @return Response
     */
    public function update(DemotableAdminWebRequest $request, Demotable $demotable)
    {
        try {

            $attributes = $request->all();

            $demotable->update($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('demotable::demotable.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/demotable/demotable/'.$demotable->getRouteKey())
            ], 201);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/demotable/demotable/'.$demotable->getRouteKey()),
            ], 400);

        }
    }

    /**
     * Remove the demotable.
     *
     * @param Model   $demotable
     *
     * @return Response
     */
    public function destroy(DemotableAdminWebRequest $request, Demotable $demotable)
    {

        try {

            $t = $demotable->delete();

            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('demotable::demotable.name')]),
                'code'     => 202,
                'redirect' => trans_url('/admin/demotable/demotable/0'),
            ], 202);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/demotable/demotable/'.$demotable->getRouteKey()),
            ], 400);
        }
    }
}
