<?php

namespace Assetdocs\Demotable\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Assetdocs\Demotable\Http\Requests\DemotableUserApiRequest;
use Assetdocs\Demotable\Interfaces\DemotableRepositoryInterface;
use Assetdocs\Demotable\Models\Demotable;

/**
 * User API controller class.
 */
class DemotableUserApiController extends BaseController
{

    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'api';

    /**
     * Initialize demotable controller.
     *
     * @param type DemotableRepositoryInterface $demotable
     *
     * @return type
     */
    public function __construct(DemotableRepositoryInterface $demotable)
    {
        $this->middleware('api');
        $this->middleware('jwt.auth:api');
        $this->setupTheme(config('theme.themes.user.theme'), config('theme.themes.user.layout'));

        $this->repository = $demotable;
        parent::__construct();
    }

    /**
     * Display a list of demotable.
     *
     * @return json
     */
    public function index(DemotableUserApiRequest $request)
    {
        $demotables  = $this->repository
            ->pushCriteria(new \Lavalite\Demotable\Repositories\Criteria\DemotableUserCriteria())
            ->setPresenter('\\Assetdocs\\Demotable\\Repositories\\Presenter\\DemotableListPresenter')
            ->scopeQuery(function($query){
                return $query->orderBy('id','DESC');
            })->all();
        $demotables['code'] = 2000;
        return response()->json($demotables) 
            ->setStatusCode(200, 'INDEX_SUCCESS');

    }

    /**
     * Display demotable.
     *
     * @param Request $request
     * @param Model   Demotable
     *
     * @return Json
     */
    public function show(DemotableUserApiRequest $request, Demotable $demotable)
    {

        if ($demotable->exists) {
            $demotable         = $demotable->presenter();
            $demotable['code'] = 2001;
            return response()->json($demotable)
                ->setStatusCode(200, 'SHOW_SUCCESS');;
        } else {
            return response()->json([])
                ->setStatusCode(400, 'SHOW_ERROR');
        }

    }

    /**
     * Show the form for creating a new demotable.
     *
     * @param Request $request
     *
     * @return json
     */
    public function create(DemotableUserApiRequest $request, Demotable $demotable)
    {
        $demotable         = $demotable->presenter();
        $demotable['code'] = 2002;
        return response()->json($demotable)
            ->setStatusCode(200, 'CREATE_SUCCESS');
    }

    /**
     * Create new demotable.
     *
     * @param Request $request
     *
     * @return json
     */
    public function store(DemotableUserApiRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.api');
            $demotable          = $this->repository->create($attributes);
            $demotable          = $demotable->presenter();
            $demotable['code']  = 2004;

            return response()->json($demotable)
                ->setStatusCode(201, 'STORE_SUCCESS');
        } catch (Exception $e) {
            return response()->json([
                'message' => $e->getMessage(),
                'code'    => 4004,
            ])->setStatusCode(400, 'STORE_ERROR');
        }

    }

    /**
     * Show demotable for editing.
     *
     * @param Request $request
     * @param Model   $demotable
     *
     * @return json
     */
    public function edit(DemotableUserApiRequest $request, Demotable $demotable)
    {
        if ($demotable->exists) {
            $demotable         = $demotable->presenter();
            $demotable['code'] = 2003;
            return response()-> json($demotable)
                ->setStatusCode(200, 'EDIT_SUCCESS');;
        } else {
            return response()->json([])
                ->setStatusCode(400, 'SHOW_ERROR');
        }
    }

    /**
     * Update the demotable.
     *
     * @param Request $request
     * @param Model   $demotable
     *
     * @return json
     */
    public function update(DemotableUserApiRequest $request, Demotable $demotable)
    {
        try {

            $attributes = $request->all();

            $demotable->update($attributes);
            $demotable         = $demotable->presenter();
            $demotable['code'] = 2005;

            return response()->json($demotable)
                ->setStatusCode(201, 'UPDATE_SUCCESS');


        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4005,
            ])->setStatusCode(400, 'UPDATE_ERROR');

        }
    }

    /**
     * Remove the demotable.
     *
     * @param Request $request
     * @param Model   $demotable
     *
     * @return json
     */
    public function destroy(DemotableUserApiRequest $request, Demotable $demotable)
    {

        try {

            $t = $demotable->delete();

            return response()->json([
                'message'  => trans('messages.success.delete', ['Module' => trans('demotable::demotable.name')]),
                'code'     => 2006
            ])->setStatusCode(202, 'DESTROY_SUCCESS');

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4006,
            ])->setStatusCode(400, 'DESTROY_ERROR');
        }
    }
}
